﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Threading;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.MainForm;

namespace HeuristicLab.HiveJobDownloader {
  public partial class HiveJobDownloaderForm : Form {

    public HiveJobDownloaderForm() {
      InitializeComponent();
      ContentManager.Initialize(new PersistenceContentManager());
      statusIcons.Images.Add(Properties.Resources.Downloading);
      statusIcons.Images.Add(Properties.Resources.Finished);
      statusIcons.Images.Add(Properties.Resources._new);
    }

    private void btnBrowse_Click(object sender, EventArgs e) {
      if (folderBrowserDialog1.ShowDialog() == DialogResult.OK) {
        this.tbxTargetDirectory.Text = folderBrowserDialog1.SelectedPath;
      }
    }

    private void refreshButton_Click(object sender, EventArgs e) {
      HiveClient.Instance.Refresh();
      try {
        jobListView.Items.Clear();
        int i = 0;
        foreach (RefreshableJob refJob in HiveClient.Instance.Jobs) {
          var cj = new HiveJob { Job = refJob, JobStatus = HiveJob.Status.New, Path = tbxTargetDirectory.Text };
          ListViewItem jobList = new ListViewItem();
          jobList.SubItems.Add(cj.Name);
          jobList.Tag = cj;
          jobList.ImageIndex = 2;
          jobListView.Items.Add(jobList);
          i++;
        }
      }
      catch (Exception E) {
        lblState.Text = E.Message;
      }
    }

    private void HiveJobDownloaderForm_Load(object sender, EventArgs e) {       
      tbxTargetDirectory.Text = @"C:\Users\Lokal\Documents";
      prbStatus.Visible = false;
      HiveClient.Instance.Refresh();
      try {
        jobListView.Items.Clear();
        jobListView.Columns.Add("Status", 50);
        jobListView.Columns.Add("Job Name", 150);
        int i = 0;
        foreach (RefreshableJob refJob in HiveClient.Instance.Jobs) {
          var cj = new HiveJob { Job = refJob, JobStatus = HiveJob.Status.New, Path = tbxTargetDirectory.Text };
          ListViewItem jobList = new ListViewItem();
          jobList.SubItems.Add(cj.Name);
          jobList.Tag = cj;
          jobList.ImageIndex = 2;
          jobListView.Items.Add(jobList);
          i++;
        }
        statusIcons.ImageSize = new Size(16, 16); 
      }
      catch (Exception E) {
        lblState.Text = E.Message;
      }
    }

    private void btnDownload_Click(object sender, EventArgs e) {
      lblStatus.Text = "Downloading ..";
      SelectedList().ImageIndex = 0;
      Thread loader = new Thread(DownloadJob);
      loader.Start();
    }

    private HiveJob SelectedJob() {
      HiveJob selectedJob = new HiveJob();
      ListViewItem lvItem = new ListViewItem();
      foreach (ListViewItem list in jobListView.SelectedItems) {
        selectedJob = (HiveJob)list.Tag;
      }
      return selectedJob;
    }

    private ListViewItem SelectedList() {
      ListViewItem lvItem = new ListViewItem();
      foreach (ListViewItem list in jobListView.SelectedItems) {
        lvItem = list;
      }
      return lvItem;
    }

    private void DownloadJob() {
      try {
        if (InvokeRequired) Invoke(new Action(DownloadJob));
        else {
          if (SelectedJob().Job.IsDownloadable) {
            prbStatus.Enabled = true;
            SelectedJob().Job.Loaded += Job_Loaded;
            SelectedJob().Job.IsProgressingChanged += Job_IsProgressingChanged;
            HiveClient.LoadJob(SelectedJob().Job);
          } else {
            lblStatus.Text = "Not Downloadable";
          }
        }
      }
      catch (Exception E) {
        lblState.Text = E.Message;
      }
    }

    void Job_IsProgressingChanged(object sender, EventArgs e) {
      var rj = (RefreshableJob)sender;
      rj.Progress.ProgressValueChanged += Progress_ProgressValueChanged;
    }


    void Progress_ProgressValueChanged(object sender, EventArgs e) {
      if (InvokeRequired) Invoke((Action<object, EventArgs>)Progress_ProgressValueChanged);
      else {
        double progressValue = ((Progress)sender).ProgressValue;
        prbStatus.Visible = true;
        prbStatus.Minimum = 0;
        prbStatus.Maximum = 100;
        prbStatus.Value = (int)Math.Round(prbStatus.Minimum + progressValue * (prbStatus.Maximum - prbStatus.Minimum));
        prbStatus.Step = 1;
        Thread update = new Thread(UpdateLblStatus);
        update.Start();
        }
    }

    private void UpdateLblStatus() {
      if (InvokeRequired) {
        MethodInvoker AssignMethodToControl = new MethodInvoker(UpdateLblStatus);
        lblStatus.Invoke(AssignMethodToControl);
      } else {
        if (prbStatus.Value < 100) {
          lblStatus.Text = "Downloading ..";
          SelectedList().ImageIndex = 0;
        } else if (prbStatus.Value == 100) {
          lblStatus.Text = "Finished";
          SelectedList().ImageIndex = 1;
          prbStatus.Visible = false;
        }
      }
    }

    void Job_Loaded(object sender, EventArgs e) {
      if (InvokeRequired) Invoke((Action<object, EventArgs>)Job_Loaded);
      else {
        RefreshableJob cJob = (RefreshableJob)sender;
        HiveJob chJob = null;

        foreach (ListViewItem list in jobListView.Items) {
          var hj = (HiveJob)list.Tag;
          if (hj.Job == cJob) {
            chJob = hj;
            break;
          }
        }

        List<HiveTask> parentTasks = new List<HiveTask>();
        foreach (var task in cJob.HiveTasks) {
          if (task.ChildHiveTasks.Count() > 0) {
            parentTasks.Add(task);
          }
        }
        if (parentTasks.Count == 0) {
          parentTasks.AddRange(cJob.HiveTasks);
        }
        int i = 0;
        foreach (var task in parentTasks) {
          string dir = Path.Combine(chJob.Path, cJob.Job.Name);
          ContentManager.Save(task.ItemTask.Item as IStorableContent, dir + i.ToString() + ".hl", true);
          chJob.JobStatus = HiveJob.Status.Downloading;
          chJob.Path = tbxTargetDirectory.Text;
          lblStatus.Text = "Downloading..";
          i++;          
        }
        SelectedList().ImageIndex = 1;
        chJob.JobStatus = HiveJob.Status.Finished;
        prbStatus.Visible = false;
      }
    }


    private void jobListView_SelectedIndexChanged(object sender, EventArgs e) {       
      if (SelectedJob().JobStatus == HiveJob.Status.New) {
        SelectedList().ImageIndex = 2;
        lblStatus.Text = SelectedJob().JobStatus.ToString();
        prbStatus.Value = 0;
        prbStatus.Visible = false;
      } else if (SelectedJob().JobStatus == HiveJob.Status.Downloading) {
        SelectedList().ImageIndex = 0;
        lblStatus.Text = "Downloading!";
        prbStatus.Visible = true; 
      } else {
        SelectedList().ImageIndex = 1;
        lblStatus.Text = SelectedJob().JobStatus.ToString();
        tbxTargetDirectory.Text = SelectedJob().Path;
        prbStatus.Visible = false;
      }
    }


  }
}
